from setuptools import setup


setup(
    name="engage",
    description="Sphinx autodoc templater",
    author="Nathan Zilora",
    author_email="zwork101@gmail.com",
    url="https://github.com/Zwork101/engage",
    license="MIT",
    packages=['engage'],
    version="1.0.1",
    install_requires=["astor"]
)
